<?php
/**
 * Simple Podcast custom fields display core php file.
 *
 * This file is a part of Simple Podcast Press and contains proprietary code.
 * Website: simplepodcastpress.com
 *
 * @package SimplePodcastPress
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Adds a custom meta box for G7G configuration.
 *
 * @return void
 */
function g7g_cfg_add_meta_box() {

	global $post;

	$wp_version = get_bloginfo( 'version' );

	$is_version_wp = version_compare( $wp_version, '4.9.8', '>' );

	$is_gutenberg_plugin = is_plugin_active( 'gutenberg/gutenberg.php' );

	if ( ! $is_version_wp && ! $is_gutenberg_plugin ) {
		return;
	}

	if ( isset( $_GET['classic-editor'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
		return;
	}

	$metadata = has_meta( $post->ID );

	if ( $metadata ) {

		foreach ( $metadata as $meta ) {
			$meta_key = isset( $meta['meta_key'] ) ? esc_attr( $meta['meta_key'] ) : null;
			$meta_val = isset( $meta['meta_value'] ) ? esc_textarea( $meta['meta_value'] ) : null;
			$meta_id  = isset( $meta['meta_id'] ) ? (int) $meta['meta_id'] : null;
			$post_id  = isset( $meta['post_id'] ) ? (int) $meta['post_id'] : null;

			if ( ! g7g_cfg_display_field( $meta_key ) ) {
				$empty = true;
			} else {
				$empty = false;
			}
		}
		if ( $empty ) {
			return;
		}
	}

	foreach ( g7g_cfg_get_post_types() as $post_type ) {

		add_meta_box( 'g7g_cfg_custom_fields', __( 'SPP Custom Fields', 'custom-fields-gutenberg' ), 'g7g_cfg_display_meta_box', $post_type, 'normal' );

	}
}

/**
 * Display the custom meta box for G7G configuration.
 *
 * This function is responsible for rendering the custom meta box content
 * for G7G configuration on the specified post.
 *
 * @param WP_Post $post The post object for which the meta box is displayed.
 *
 * @return void
 */
function g7g_cfg_display_meta_box( $post ) {

	$metadata = has_meta( $post->ID );

	if ( $metadata ) {

		echo '<div class="g7g-meta-box">';

		if ( g7g_cfg_display_row_table( $metadata ) ) {

			echo '<table>';
			echo '<thead>';
			echo '<tr>';
			echo '<th>' . esc_html__( 'Name', 'custom-fields-gutenberg' ) . '</th>';
			echo '<th>' . esc_html__( 'Value', 'custom-fields-gutenberg' ) . '</th>';
			echo '</tr>';
			echo '</thead>';
			echo '<tbody>';

			foreach ( $metadata as $meta ) {
				echo g7g_cfg_list_meta_row( $meta ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			}
			echo '</tbody>';
			echo '</table>';
		}
		echo g7g_cfg_meta_form( $post ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo '</div>';
	}
}

/**
 * Display a row in the G7G meta list based on the provided meta data.
 *
 * This function is responsible for rendering a single row in the G7G meta list
 * based on the given meta data.
 *
 * @param array $meta An array containing meta information for the row.
 *
 * @return string Return html string.
 */
function g7g_cfg_list_meta_row( $meta ) {

	$meta_key = isset( $meta['meta_key'] ) ? esc_attr( $meta['meta_key'] ) : null;
	$meta_val = isset( $meta['meta_value'] ) ? esc_textarea( $meta['meta_value'] ) : null;
	$meta_id  = isset( $meta['meta_id'] ) ? (int) $meta['meta_id'] : null;
	$post_id  = isset( $meta['post_id'] ) ? (int) $meta['post_id'] : null;

	if ( empty( $meta_key ) ) {

		delete_metadata_by_mid( 'post', $meta_id );

		return '';

	}

	if ( ! g7g_cfg_display_protected( $meta_key ) ) {
		return '';
	}

	if ( ! g7g_cfg_display_empty( $meta_val ) ) {
		return '';
	}

	if ( ! g7g_cfg_display_field( $meta_key ) ) {
		return '';
	}

	if ( is_serialized( $meta_val ) ) {

		if ( is_serialized_string( $meta_val ) ) {
			$meta_val = maybe_unserialize( $meta_val );

		} else {
			return '';
		}
	}

	$r = '<tr id="meta-' . $meta_id . '">';

	$r .= '<td><label class="screen-reader-text" for="meta-' . $meta_id . '-key">' . __( 'Key', 'custom-fields-gutenberg' ) . '</label>';

	$r .= '<input name="meta[' . $meta_id . '][key]" id="meta-' . $meta_id . '-key" disable type="text" size="20" value="' . $meta_key . '" disabled="disabled" />';

	$r .= wp_nonce_field( 'change-meta', '_ajax_nonce', false, false );

	$r .= '</td>';

	$r .= '<td><label class="screen-reader-text" for="meta-' . $meta_id . '-value">' . __( 'Value', 'custom-fields-gutenberg' ) . '</label>';

	$r .= '<textarea name="meta[' . $meta_id . '][value]" id="meta-' . $meta_id . '-value" rows="2" cols="30">' . $meta_val . '</textarea>';

	$r .= '</td>';

	$r .= '</tr>';

	return $r;
}

/**
 * Render the G7G configuration meta form.
 *
 * This function is responsible for generating and displaying the HTML form
 * for G7G configuration meta data.
 *
 * @param WP_Post|null $post Optional. The post object for which the meta form is rendered.
 *                           If null, it's considered a new post.
 *
 * @return void
 */
function g7g_cfg_meta_form( $post = null ) {

	global $wpdb;

	$post = get_post( $post );

	$keys = apply_filters( 'postmeta_form_keys', null, $post );

	if ( null === $keys ) {

		$limit = apply_filters( 'postmeta_form_limit', 30 );

		$sql = "SELECT DISTINCT meta_key 
				FROM $wpdb->postmeta 
				WHERE meta_key NOT BETWEEN '_' AND '_z' 
				HAVING meta_key NOT LIKE %s 
				ORDER BY meta_key 
				LIMIT %d";

		$keys = $wpdb->get_col( $wpdb->prepare( $sql, $wpdb->esc_like( '_' ) . '%', $limit ) ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching,WordPress.DB.PreparedSQL.NotPrepared

	}

	if ( $keys ) {

		natcasesort( $keys );

		$meta_key_input_id = 'metakeyselect';

	} else {

		$meta_key_input_id = 'metakeyinput';

	}

	?>
	
	<?php
}

/**
 * Display a row table for G7G configuration based on the provided metadata.
 *
 * This function is responsible for rendering and displaying a row table
 * for G7G configuration using the provided metadata.
 *
 * @param array $metadata An array containing metadata for the row table.
 *
 * @return void
 */
function g7g_cfg_display_row_table( $metadata ) {

	$exclude_protected = 0;
	$exclude_empty     = 0;

	$display = false;

	if ( ! is_array( $metadata ) ) {
		return false;
	}

	foreach ( $metadata as $meta ) {

		$meta_key = isset( $meta['meta_key'] ) ? $meta['meta_key'] : null;
		$meta_val = isset( $meta['meta_value'] ) ? $meta['meta_value'] : null;

		if ( substr( $meta_key, 0, 1 ) !== '_' ) {

			if ( ! empty( $meta_val ) ) {

				$display = true;
				break;

			} elseif ( empty( $meta_val ) && ! $exclude_empty ) {

				$display = true;
				break;

			}
		} elseif ( ! $exclude_protected ) {

			if ( ! empty( $meta_val ) ) {

				$display = true;
				break;

			} elseif ( empty( $meta_val ) && ! $exclude_empty ) {

				$display = true;
				break;

			}
		}
	}

	return $display;
}

/**
 * Retrieve an array of post types relevant to G7G configuration.
 *
 * This function returns an array containing the post types that are relevant
 * to G7G configuration settings.
 *
 * @return array An array of post types.
 */
function g7g_cfg_get_post_types() {

	$post_types = get_post_types( array(), 'objects' );

	$unset = array( 'attachment', 'revision', 'nav_menu_item', 'custom_css', 'customize_changeset', 'oembed_cache' );

	$unset = apply_filters( 'g7g_cfg_post_types_unset', $unset );

	$types = array();

	foreach ( $post_types as $key => $post_type ) {

		$types[ $key ]['name']  = $post_type->name;
		$types[ $key ]['label'] = $post_type->label;

		if ( in_array( $post_type->name, $unset, true ) || ! post_type_supports( $post_type->name, 'custom-fields' ) ) {
			unset( $types[ $key ] );
		}
	}

	return apply_filters( 'g7g_cfg_get_post_types', $types );
}


/**
 * Determine if a meta key should be displayed as protected.
 *
 * This function checks whether the given meta key should be displayed as protected.
 *
 * @param string $meta_key The meta key to check.
 *
 * @return bool True if the meta key should be displayed as protected, false otherwise.
 */
function g7g_cfg_display_protected( $meta_key ) {
	return true;
}

/**
 * Determine if a meta value should be displayed as empty.
 *
 * This function checks whether the given meta value should be displayed as empty.
 *
 * @return bool True if the meta value should be displayed as empty, false otherwise.
 */
function g7g_cfg_display_empty() {
	return true;
}

/**
 * Determine if a meta key should be displayed as a field.
 *
 * This function checks whether the given meta key should be displayed as a field.
 *
 * @param string $meta_key The meta key to check.
 *
 * @return bool True if the meta key should be displayed as a field, false otherwise.
 */
function g7g_cfg_display_field( $meta_key ) {
	return $meta_key === '_audiourl' || $meta_key === '_useraudiourl'; // phpcs:ignore WordPress.PHP.YodaConditions.NotYoda
}

/**
 * Generate a random string of a specified length.
 *
 * This function generates a random string composed of alphanumeric characters.
 *
 * @param int $length The length of the random string (default is 10).
 *
 * @return string The generated random string.
 */
function g7g_cfg_random_string( $length = 10 ) {
	$characters        = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
	$characters_length = strlen( $characters );
	$random_string     = '';

	for ( $i = 0; $i < $length; $i++ ) {
		$random_string .= $characters[wp_rand( 0, $characters_length - 1 )];
	}

	return $random_string;
}


