<?php // phpcs:ignore WordPress.Files.FileName.InvalidClassFileName

add_shortcode( 'spp-reviews', 'spp_reviews_shortcode' );
add_action( 'widgets_init', 'spp_reviews_load' );

/**
 * Generates HTML content for the 'spp_reviews' shortcode.
 *
 * This function is a callback for the 'spp_reviews' shortcode and generates HTML content based on the
 * provided attributes.
 *
 * @param array $atts Shortcode attributes provided by the user.
 * @return string HTML content for the 'spp_reviews' shortcode.
 */
function spp_reviews_shortcode( $atts ) {
	extract(
		shortcode_atts(
			array(
				'max'   => 999,
				'style' => 'slideshow',
			),
			$atts
		)
	);
	wp_enqueue_script( 'rotator', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/rotator.js', array( 'jquery' ), 1, true );
	return show_reviews( $max, $style );
}

/**
 * Displays reviews based on specified limits and format.
 *
 * This function retrieves and returns reviews with the provided limit and format as HTML content.
 *
 * @param int    $limit  The maximum number of reviews to retrieve.
 * @param string $format The format in which to display the reviews.
 * @return string HTML content of the reviews.
 */
function show_reviews( $limit, $format ) {
				$reviews = get_reviews( $limit, $format );

	if ( empty( $reviews ) ) {
		return;
	}
			$html = '<div class="reviews_view"><ul  class="reviews_view" style="' . $format . '" current="0">';
	foreach ( $reviews as $review ) {
		$html .= '<li style="list-style:none !important">';
		$html .= '<strong>' . $review['title'] . '</strong>';
		$html .= '<img src="' . SPPRESS_PLUGIN_URL . '/icons/rating_stars/' . $review['ratings'] . 'star.png">';

		$published_date = gmdate( 'F j, Y', strtotime( $review['published_date'] ) );
		$html          .= '<div>' . $published_date . ' by  <b>' . $review['author'] . '</b> from ' . get_full_country_name( $review['country'] ) . '</div>';
		$html          .= '<p>' . $review['text'] . '</p>';
		$html          .= '</li>';
	}
			$html .= '</ul></div>';
			return $html;
}

/**
 * Retrieves reviews based on specified limits and format.
 *
 * This function retrieves reviews with the provided limit and format.
 *
 * @param int $limit  The maximum number of reviews to retrieve.
 * @return string Reviews data as a string. Format may vary based on the provided format parameter.
 */
function get_reviews( $limit ) {

	$ret  = array();
	$item = array();
	global $wpdb;
	$table_name = $wpdb->prefix . 'spp_reviews';
	if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_name'" ) !== $table_name ) { // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
		return;
	}

	$reviews = $wpdb->get_results( "select * from $table_name  where rw_ratings='5' OR rw_ratings='4' ORDER BY rw_id DESC limit $limit" ); // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching

	foreach ( $reviews as $review ) {
		$item['title']          = $review->rw_title;
		$item['text']           = $review->rw_text;
		$item['ratings']        = $review->rw_ratings;
		$item['author']         = $review->rw_author;
		$item['published_date'] = $review->rw_published_date;
		$item['author']         = $review->rw_author;
		$item['country']        = $review->rw_coutry;
		$ret[]                  = $item;
	}
	return $ret;
}

/**
 * Widget for displaying reviews.
 *
 * This class extends the WP_Widget class to create a widget for displaying reviews.
 *
 * @package YourPackage
 */
class SppReviewsWidget extends WP_Widget { // phpcs:ignore Universal.Files.SeparateFunctionsFromOO.Mixed

	/**
	 * Constructor method for the SppReviewsWidget class.
	 *
	 * Initializes the widget with the specified parameters.
	 *
	 * @see WP_Widget::__construct() For more details on parameters.
	 */
	public function __construct() {
		parent::__construct( 'sppreviews_widget', 'Podcast Reviews', array( 'description' => 'Showcase Your Podcast Reviews' ) );
	}

	/**
	 * Outputs the content of the widget.
	 *
	 * This method is called to generate the output of the widget when it is displayed on the site.
	 *
	 * @param array $args     An array of widget arguments.
	 * @param array $instance An array of widget instance settings.
	 */
	public function widget( $args, $instance ) {
		$title = 'Podcast Reviews';

		$limit  = apply_filters( 5, $instance['max'] );
		$format = apply_filters( 'list', $instance['style'] );
		echo $args['before_widget'];
		if ( ! empty( $limit ) ) {
			echo $args['before_title'] . $title . $args['after_title'];
		}

		wp_enqueue_script( 'rotator', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/rotator.js', array( 'jquery' ), false, true );

		echo show_reviews( $limit, $format ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo $args['after_widget'];
	}

	/**
	 * Outputs the settings form for the widget.
	 *
	 * This method is called to generate the settings form for the widget in the WordPress admin area.
	 *
	 * @param array $instance An array of widget instance settings.
	 */
	public function form( $instance ) {
		if ( isset( $instance['max'] ) ) {
			$limit = $instance['max'];
		} else {
			$limit = 5;
		}
		if ( isset( $instance['style'] ) ) {
			$format = $instance['style'];
		} else {
			$format = 'list';
		}
		?>

			<table>
				<tr>
					<td>Number of Reviews:</td>
					<td><input style="width: 75px;" type="text" value="<?php echo esc_attr( $limit ); ?>" name="<?php echo $this->get_field_name( 'max' ); ?>" id="<?php echo $this->get_field_id( 'max' ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>"></td> 
				</tr>
				<tr>
					<td>Style:</td>                       
					<td><input type="radio" value="list" id="<?php echo $this->get_field_id( 'style' ); ?>" name="<?php echo $this->get_field_name( 'style' ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>" 
					<?php
					if ( 'list' === $format ) {
						echo 'checked';
					}
					?>
					>List</td>
					<td><input type="radio" value="slideshow" id="<?php echo $this->get_field_id( 'style' ); ?>" name="<?php echo $this->get_field_name( 'style' ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>" 
					<?php
					if ( 'list' !== $format ) {
							echo 'checked';
					}
					?>
					>Slideshow</td>
				</tr>
			</table><br>
		<?php
	}
	/**
	 * Processes and saves widget settings when updated.
	 *
	 * This method is called to process and save the widget settings when it is updated in the WordPress admin area.
	 *
	 * @param array $newInstance The new instance settings.
	 * @param array $oldInstance The old instance settings.
	 * @return array Updated instance settings to be saved.
	 */
	public function update( $newInstance, $oldInstance ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$instance          = array();
		$instance['max']   = ( ! empty( $newInstance['max'] ) ) ? wp_strip_all_tags( $newInstance['max'] ) : 5; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$instance['style'] = ( ! empty( $newInstance['style'] ) ) ? wp_strip_all_tags( $newInstance['style'] ) : 'list'; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		return $instance;
	}
}

/**
 * Registers the SppReviewsWidget widget for use.
 *
 * This function registers the SppReviewsWidget widget, making it available for use in the WordPress
 * admin area and on the site.
 */
function spp_reviews_load() {
	register_widget( 'SppReviewsWidget' );
}

/**
 * Retrieves the full name of a country based on its short code.
 *
 * This function takes a short code (e.g., "US") and returns the corresponding full name of the country
 * (e.g., "United States").
 *
 * @param string $short The short code of the country.
 * @return string The full name of the country.
 */
function get_full_country_name( $short ) {
	switch ( $short ) {
		case 'us':
			$fullname = 'United States';
			break;
		case 'ca':
			$fullname = 'Canada';
			break;
		case 'la':
			$fullname = 'Puerto Rico"';
			break;
		case 'mx':
			$fullname = 'Mexico';
			break;
		case 'gb':
			$fullname = 'United Kingdom';
			break;
		case 'br':
			$fullname = 'Brazil';
			break;
		case 'tr':
			$fullname = 'Turkey';
			break;
		case 'se':
			$fullname = 'Sweden';
			break;
		case 'fi':
			$fullname = 'Finland';
			break;
		case 'ch':
			$fullname = 'Switzerland';
			break;
		case 'si':
			$fullname = 'Slovenia';
			break;
		case 'at':
			$fullname = 'Austria';
			break;
		case 'pl':
			$fullname = 'Poland';
			break;
		case 'pt':
			$fullname = 'Portugal';
			break;
		case 'ro':
			$fullname = 'Romania';
			break;
		case 'ru':
			$fullname = 'Russia';
			break;
		case 'sk':
			$fullname = 'Slovak Republic';
			break;
		case 'no':
			$fullname = 'Norway';
			break;
		case 'nl':
			$fullname = 'Netherlands';
			break;
		case 'md':
			$fullname = 'Moldova';
			break;
		case 'mt':
			$fullname = 'Malta';
			break;
		case 'hu':
			$fullname = 'Hungary';
			break;
		case 'mk':
			$fullname = 'Macedonia';
			break;
		case 'lu':
			$fullname = 'Luxembourg';
			break;
		case 'lt':
			$fullname = 'Lithuania';
			break;
		case 'lv':
			$fullname = 'Latvia';
			break;
		case 'it':
			$fullname = 'Italy';
			break;
		case 'ie':
			$fullname = 'Ireland';
			break;
		case 'hr':
			$fullname = 'Croatia';
			break;
		case 'gr':
			$fullname = 'Greece';
			break;
		case 'fr':
			$fullname = 'France';
			break;
		case 'es':
			$fullname = 'Spain';
			break;
		case 'ee':
			$fullname = 'Estonia';
			break;
		case 'de':
			$fullname = 'Germany';
			break;
		case 'dk':
			$fullname = 'Denmark';
			break;
		case 'cz':
			$fullname = 'Czech Republic';
			break;
		case 'bg':
			$fullname = 'Bulgaria';
			break;
		case 'be':
			$fullname = 'Belgium';
			break;
		case 'vn':
			$fullname = 'Vietnam';
			break;
		case 'th':
			$fullname = 'Thailand';
			break;
		case 'tw':
			$fullname = 'Taiwan';
			break;
		case 'sg':
			$fullname = 'Singapore';
			break;
		case 'ph':
			$fullname = 'Philippines';
			break;
		case 'nz':
			$fullname = 'New Zealand';
			break;
		case 'my':
			$fullname = 'Malaysia';
			break;
		case 'kr':
			$fullname = 'South Korea';
			break;
		case 'jp':
			$fullname = 'Japan';
			break;
		case 'id':
			$fullname = 'Indonesia';
			break;
		case 'hk':
			$fullname = 'Hong Kong';
			break;
		case 'cn':
			$fullname = 'China';
			break;
		case 'au':
			$fullname = 'Australia';
			break;
		case 'ae':
			$fullname = 'United Arab Emirates';
			break;
		case 'ug':
			$fullname = 'Uganda';
			break;
		case 'tn':
			$fullname = 'Tunisia';
			break;
		case 'za':
			$fullname = 'South Africa';
			break;
		case 'sn':
			$fullname = 'Senegal';
			break;
		case 'sa':
			$fullname = 'Saudi Arabia';
			break;
		case 'om':
			$fullname = 'Oman';
			break;
		case 'ng':
			$fullname = 'Nigeria';
			break;
		case 'ne':
			$fullname = 'Niger';
			break;
		case 'mz':
			$fullname = 'Mozambique';
			break;
		case 'mu':
			$fullname = 'Mauritius';
			break;
		case 'ml':
			$fullname = 'Mali';
			break;
		case 'mg':
			$fullname = 'Madagascar';
			break;
		case 'kw':
			$fullname = 'Kuwait';
			break;
		case 'ke':
			$fullname = 'Kenya';
			break;
		case 'jo':
			$fullname = 'Jordan';
			break;
		case 'il':
			$fullname = 'Israel';
			break;
		case 'in':
			$fullname = 'India';
			break;
		case 'gw':
			$fullname = 'Guinea Bissau';
			break;
		case 'eg':
			$fullname = 'Egypt';
			break;
		case 'bw':
			$fullname = 'Botswana';
			break;
		case 'bh':
			$fullname = 'Bahrain';
			break;
		case 'am':
			$fullname = 'Armenia';
			break;

	}//end switch
	return $fullname;
}