<?php // phpcs:ignore WordPress.Files.FileName.InvalidClassFileName
/**
 * Simple Podcast click to tweet function File
 *
 * This file is a part of Simple Podcast Press and contains proprietary code.
 * Website: simplepodcastpress.com
 *
 * @package SimplePodcastPress
 */

if ( ! class_exists( 'Spp_clicktotweet' ) ) {

	/**
	 * Class Spp_clicktotweet
	 *
	 * This class handles the functionality related to Click to Tweet in the Simple Podcast Press plugin.
	 *
	 * @package SimplePodcastPress
	 */
	class Spp_clicktotweet { // phpcs:ignore Generic.Classes.OpeningBraceSameLine.ContentAfterBrace, phpcs:ignore PEAR.NamingConventions.ValidClassName.StartWithCapital, PEAR.NamingConventions.ValidClassName.Invalid

		/**
		 * Class constructor: initializes class variables and adds actions and filters.
		 *
		 * @constructor
		 */
		public function __construct() {

			$this->spp_clicktotweet();
			add_shortcode( 'spp-tweet', array( $this, 'spp_clicktotweet_params' ) );
		}


		/**
		 * Retrieves parameters for the Click to Tweet functionality.
		 *
		 * This method is used to extract parameters passed through the shortcode for Click to Tweet.
		 *
		 * @param array $atts Shortcode attributes.
		 *
		 * @return array An array of parameters for the Click to Tweet functionality.
		 */
		public function spp_clicktotweet_params( $atts ) {
			if ( isset( $atts['tweet'] ) ) {
				global $wpdb;
		
				$handle_code = '';
				$spp_episode = '';
				$table_spp_links = $wpdb->prefix . 'spp_links';
				$handle = Wp_Simplepodcastpress::spp_get_option( 'spp-twitter-handle' );
		
				if ( ! empty( $handle ) ) {
					$handle_code = '&via=' . $handle . '&related=' . $handle;
				}
		
				$text = $atts['tweet'];
				$short = $this->shorten( $text, 100 );
				$tweetshort = $short . ' - ';
				$disable_url_shortner = Wp_Simplepodcastpress::spp_get_option( 'disable_url_shortner' );
		
				if ( $disable_url_shortner ) {
					$url = wp_get_shortlink();
				} else {
					global $post;
					setup_postdata( $post );
					$post_id = get_the_ID();
					$spp_slug = $wpdb->get_row(
						$wpdb->prepare(
							"SELECT spp_slug FROM {$table_spp_links} WHERE spp_post_id = %d",
							$post_id
						)
					);
		
					if ( isset( $spp_slug->spp_slug ) ) {
						$url = get_site_url() . '/' . $spp_slug->spp_slug;
					} else {
						$url = wp_get_shortlink();
					}
				}
		
				return "<div class='spp-tweet-clear'></div><div class='spp-click-to-tweet'><div class='spp-ctt-text'><a href='https://twitter.com/share?text=" . rawurlencode( $short ) . $handle_code . '&url=' . $url . "' target='_blank'>" . $text . "</a></div><a href='https://twitter.com/share?text=" . rawurlencode( $tweetshort ) . '' . $handle_code . '&url=' . $url . "' target='_blank' class='spp-ctt-btn'>Tweet This</a><div class='spp-ctt-tip'></div></div>";
			}
		}
		

		/**
		 * Registers activation and deactivation hooks, and global Click to Tweet hooks.
		 *
		 * This method is called when the class is instantiated and performs the following:
		 * - Registers activation and deactivation hooks for the plugin.
		 * - Registers global hooks for the Click to Tweet functionality.
		 *
		 * @return void
		 */
		public function spp_clicktotweet() {

			register_activation_hook( __FILE__, array( __CLASS__, 'activation' ) );

			register_deactivation_hook( __FILE__, array( __CLASS__, 'deactivation' ) );

			// Register global hooks.
			$this->register_global_hooks();
		}


		/**
		 * Handles activation tasks, such as registering the uninstall hook.
		 *
		 * This method is called when the plugin is activated and performs tasks specific to activation,
		 * such as registering the uninstall hook to handle cleanup on plugin uninstallation.
		 *
		 * @return void
		 */
		public function activation() {
			register_uninstall_hook( __FILE__, array( __CLASS__, 'uninstall' ) );
		}



		/**
		 * Handles deactivation tasks, such as deleting plugin options.
		 *
		 * This method is called when the plugin is deactivated and performs tasks specific to deactivation,
		 * such as cleaning up or deleting plugin options to ensure a smooth deactivation process.
		 *
		 * @return void
		 */
		public function deactivation() {
		}



		/**
		 * Handles uninstallation tasks, such as deleting plugin options.
		 *
		 * This method is called when the plugin is being uninstalled and performs tasks specific to uninstallation,
		 * such as deleting plugin options to ensure a clean uninstallation process.
		 *
		 * @return void
		 */
		public function uninstall() {
			delete_option( 'spp-twitter-handle' );
		}

		/**
		 * Registers global hooks, these are added to both the admin and front-end.
		 *
		 * This method is responsible for registering global hooks that are applied to both the admin and front-end sections.
		 * These hooks play a role in the overall functionality of the plugin.
		 *
		 * @return void
		 */
		public function register_global_hooks() {

			if ( is_admin() ) {

				// Cache bust tinymce.

				add_filter( 'tiny_mce_version', array( $this, 'refresh_mce' ) );

				// Add button plugin to TinyMCE.

				add_action( 'init', array( $this, 'tinymce_button' ) );

			} else {
				add_filter( 'the_content', array( $this, 'replace_tags' ), 1 );
			}
		}

		/**
		 * Adds a TinyMCE button for the plugin.
		 *
		 * This method checks the user's capability to edit posts or pages before adding
		 * a TinyMCE button for the plugin. If the user lacks the necessary capabilities,
		 * the method returns early.
		 *
		 * @return void
		 */
		public function tinymce_button() {

			if ( ! current_user_can( 'edit_posts' ) && ! current_user_can( 'edit_pages' ) ) {

				return;

			}

			if ( 'true' === get_user_option( 'rich_editing' ) ) {

				add_filter( 'mce_external_plugins', array( $this, 'tinymce_register_plugin' ) );
				add_filter( 'mce_buttons_3', array( $this, 'tinymce_register_button' ) );
				add_filter( 'mce_buttons_3', array( $this, 'tinymce_register_button_spptweet' ) );
				add_filter( 'mce_external_plugins', array( $this, 'tinymce_register_plugin_spptweet' ) );

			}
		}

		/**
		 * Registers the TinyMCE button 'spptweet'.
		 *
		 * This method adds the 'spptweet' button to the array of TinyMCE buttons.
		 *
		 * @param array $buttons Array of TinyMCE buttons.
		 * @return array Modified array of TinyMCE buttons.
		 */
		public function tinymce_register_button_spptweet( $buttons ) {
			array_push( $buttons, '|', 'spptweet' );
			return $buttons;
		}

		/**
		 * Registers a custom TinyMCE button.
		 *
		 * This method allows registering a custom TinyMCE button by adding it to the array of buttons.
		 *
		 * @param array $buttons Array of TinyMCE buttons.
		 * @return array Modified array of TinyMCE buttons.
		 */
		public function tinymce_register_button( $buttons ) {

			array_push( $buttons, '|', 'sppplayer' );

			array_push( $buttons, '|', 'sppplaylist' );

			array_push( $buttons, '|', 'sppoptin' );

			array_push( $buttons, '|', 'spptranscript' );

			array_push( $buttons, '|', 'spptranscriptclose' );

			array_push( $buttons, '|', 'sppctabuttons' );

			array_push( $buttons, '|', 'spptimestamp' );

			array_push( $buttons, '|', 'sppepisodes' );

			array_push( $buttons, '|', 'sppreviews' );

			array_push( $buttons, '|', 'sppfeaturedImage' );

			return $buttons;
		}

		/**
		 * Registers the TinyMCE plugin 'spptweet'.
		 *
		 * This method adds the 'spptweet' TinyMCE plugin to the array of registered plugins.
		 *
		 * @param array $plugin_array Array of registered TinyMCE plugins.
		 * @return array Modified array of registered TinyMCE plugins.
		 */
		public function tinymce_register_plugin_spptweet( $plugin_array ) {
			$plugin_array['spptweet'] = plugins_url( '/assets/js/spptweet_plugin.js', __FILE__ );
			return $plugin_array;
		}

		/**
		 * Registers a custom TinyMCE plugin.
		 *
		 * This method allows registering a custom TinyMCE plugin by adding it to the array of registered plugins.
		 *
		 * @param array $plugin_array Array of registered TinyMCE plugins.
		 * @return array Modified array of registered TinyMCE plugins.
		 */
		public function tinymce_register_plugin( $plugin_array ) {

			$plugin_array['sppplayer'] = plugins_url( '/assets/js/sppplayer_plugin.js', __FILE__ );

			$plugin_array['sppplaylist'] = plugins_url( '/assets/js/sppplaylist_plugin.js', __FILE__ );

			$plugin_array['sppoptin'] = plugins_url( '/assets/js/sppoptin_plugin.js', __FILE__ );

			$plugin_array['spptranscript'] = plugins_url( '/assets/js/spptranscript_plugin.js', __FILE__ );

			$plugin_array['spptranscriptclose'] = plugins_url( '/assets/js/spptranscriptclose_plugin.js', __FILE__ );

			$plugin_array['sppctabuttons'] = plugins_url( '/assets/js/sppctabuttons_plugin.js', __FILE__ );

			$plugin_array['spptimestamp'] = plugins_url( '/assets/js/spptimestamp_plugin.js', __FILE__ );

			$plugin_array['sppepisodes'] = plugins_url( '/assets/js/sppepisodes_plugin.js', __FILE__ );

			$plugin_array['sppreviews'] = plugins_url( '/assets/js/sppreviews_plugin.js', __FILE__ );

			$plugin_array['sppfeaturedImage'] = plugins_url( '/assets/js/sppfeaturedimage_plugin.js', __FILE__ );

			return $plugin_array;
		}



		/**
		 * Adds a settings link to the plugin management page in the admin area.
		 *
		 * This method is a callback for the 'plugin_action_links_' filter and adds a settings link to the
		 * list of actions for the plugin on the plugin management page.
		 *
		 * @param array  $actions An array of plugin action links.
		 * @param string $file    The plugin file path.
		 * @return array Modified array of plugin action links.
		 */
		public function plugin_settings_link( $actions, $file ) {

			if ( false !== strpos( $file, 'spp-tweet-click-to-tweet' ) ) {

				$actions['settings'] = '<a href="admin.php?page=spp-podcast-settings">Settings</a>';

			}

			return $actions;
		}

		/**
		 * Validates settings submitted in the admin area.
		 *
		 * This method is a callback for settings validation and is called when settings are submitted
		 * in the admin area. It validates the input data before saving it.
		 *
		 * @param array $input The array of input data submitted in the admin area.
		 * @return array Validated array of settings.
		 */
		public function validate_settings( $input ) {

			return str_replace( '@', '', wp_strip_all_tags( stripslashes( $input ) ) );
		}

		/**
		 * Shortens text length to a specified number of characters.
		 *
		 * This method takes an input text and shortens it to the specified length, adding ellipses
		 * if desired. HTML tags can be optionally stripped from the input text before shortening.
		 *
		 * @param string $input      The input text to be shortened.
		 * @param int    $length     The desired length of the shortened text.
		 * @param bool   $ellipses   Whether to add ellipses (...) if the text is shortened.
		 * @param bool   $strip_html Whether to strip HTML tags from the input text before shortening.
		 * @return string Shortened text.
		 */
		public function shorten( $input, $length, $ellipses = true, $strip_html = true ) {

			if ( $strip_html ) {
				$input = wp_strip_all_tags( $input );
			}

			if ( strlen( $input ) <= $length ) {
				return $input;
			}

			$last_space = strrpos( substr( $input, 0, $length ), ' ' );

			$trimmed_text = substr( $input, 0, $last_space );

			if ( $ellipses ) {

				$trimmed_text .= '...';

			}

			return $trimmed_text;
		}

		/**
		 * Replaces Tweet tags with the correct HTML.
		 *
		 * This method is a callback function for a preg_replace_callback and is used to replace Tweet tags
		 * with the correct HTML content.
		 *
		 * @param array $matches An array of matches from the preg_replace_callback.
		 * @return string Correct HTML content to replace the Tweet tags.
		 */
		public function tweet( $matches ) {

			global $post;

			global $wpdb;

			$handle_code = '';

			$spp_episode = '';

			$table_spp_links = $wpdb->prefix . 'spp_links';

			$handle = Wp_Simplepodcastpress::spp_get_option( 'spp-twitter-handle' );

			if ( ! empty( $handle ) ) {

				$handle_code = '&via=' . $handle . '&related=' . $handle;

			}

			$text = $matches[1];

			$short = $this->shorten( $text, 100 );

			$tweetshort = $short . ' - ';

			$url = get_site_url();

			$disable_url_shortner = Wp_Simplepodcastpress::spp_get_option( 'disable_url_shortner' );

			if ( $disable_url_shortner ) {
				$url = wp_get_shortlink();
			} else {
				$spp_slug = $wpdb->get_row( $wpdb->prepare(
					"SELECT spp_slug FROM {$table_spp_links} WHERE spp_post_id = %d", // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
					$post->ID )
				);
				if ( isset( $spp_slug ) ) {
					$url = $url . '/' . $spp_slug->spp_slug;
				} else {
					$url = wp_get_shortlink();
				}
			}

			return "<div class='spp-tweet-clear'></div><div class='spp-click-to-tweet'><div class='spp-ctt-text'><a href='https://twitter.com/share?text=" . rawurlencode( $short ) . $handle_code . '&url=' . $url . "' target='_blank'>" . $text . "</a></div><a href='https://twitter.com/share?text=" . rawurlencode( $tweetshort ) . '' . $handle_code . '&url=' . $url . "' target='_blank' class='spp-ctt-btn'>Tweet This</a><div class='spp-ctt-tip'></div></div>";
		}



		/**
		 * Replaces Tweet tags with the correct HTML for an RSS feed.
		 *
		 * This method is a callback function for a preg_replace_callback and is specifically used to replace
		 * Tweet tags with the correct HTML content in the context of an RSS feed.
		 *
		 * @param array $matches An array of matches from the preg_replace_callback.
		 * @return string Correct HTML content to replace the Tweet tags in the RSS feed.
		 */
		public function tweet_feed( $matches ) {

			$handle_code = '';

			$handle = Wp_Simplepodcastpress::spp_get_option( 'spp-twitter-handle' );

			if ( ! empty( $handle ) ) {

				$handle_code = '&via=' . $handle . '&related=' . $handle;

			}

			$text = $matches[1];

			$short = $this->shorten( $text, 100 );

			$tweetshort = $short . ' - ';

			return '<hr /><p><em>' . $short . "</em><br /><a href='https://twitter.com/share?text=" . rawurlencode( $tweetshort ) . $handle_code . '&url=' . wp_get_shortlink() . "' target='_blank'>Tweet This</a></p><hr />";
		}

		/**
		 * Uses a regular expression to locate Tweet tags in the content and replace them.
		 *
		 * This method searches for Tweet tags in the provided content using a regular expression and replaces
		 * them as needed.
		 *
		 * @param string $content The content in which Tweet tags are to be replaced.
		 * @return string Content with Tweet tags replaced.
		 */
		public function replace_tags( $content ) {

			if ( ! is_feed() ) {

				$content = preg_replace_callback( '/\[spp-tweet "(.*?)"]/i', array( $this, 'tweet' ), $content );

			} else {

				$content = preg_replace_callback( '/\[tweet "(.*?)"]/i', array( $this, 'tweet_feed' ), $content );

			}

			return $content;
		}

		/**
		 * Performs cache busting for TinyMCE by modifying the version number.
		 *
		 * This method is a callback function for the 'tiny_mce_before_init' filter and is used to perform cache
		 * busting for TinyMCE by modifying the version number.
		 *
		 * @param string $ver The current version number of TinyMCE.
		 * @return string Modified version number for cache busting.
		 */
		public function refresh_mce( $ver ) {

			$ver += 3;

			return $ver;
		}
	} // End spp_clicktotweet class.

	// Init Class.
	new Spp_clicktotweet();

}
