<?php
/**
 * File spp-update.php
 *
 * This file is a part of Simple Podcast Press and contains proprietary code.
 * Website: simplepodcastpress.com
 *
 * @package SimplePodcastPress
 */

/**
 * Register the Simple Podcast Press License submenu page.
 *
 * This function adds a submenu page under 'spp-podcast-settings' in the WordPress admin menu.
 * The submenu page is titled 'License' and allows users with 'manage_options' capability to access it.
 * The page callback function for the submenu page is 'edd_sppress_license_page'.
 */
function edd_sppress_license_menu() {
	add_submenu_page( 'spp-podcast-settings', 'Simple Podcast Press', 'License', 'manage_options', 'spp-license', 'edd_sppress_license_page' );
}
add_action( 'admin_menu', 'edd_sppress_license_menu' );

/**
 * Display the content for the Simple Podcast Press License submenu page.
 *
 * This function is used as the callback for the 'spp-license' submenu page. It is responsible for
 * displaying the content on the page.
 */
function edd_sppress_license_page() { // phpcs:ignore Squiz.Commenting.ClosingDeclarationComment.Abstract
	$license_key = get_option( 'edd_sppress_license_key' ); // phpcs:ignore PSR12.Functions.ReturnTypeDeclaration.SpaceBeforeReturnType
	$status      = get_option( 'sppress_ls' );

	$check_expiry = array(
		'edd_action' => 'check_license',
		'license'    => $license_key,
		'item_name'  => rawurlencode( SPPRESS_ITEM_NAME ),
		'url'        => get_site_url(),
	);
	$response     = wp_remote_post(
		SPPRESS_STORE_URL,
		array(
			'timeout' => 60,
			'body'    => $check_expiry,
		)
	);

	$update_data = json_decode( wp_remote_retrieve_body( $response ) );
	if ( isset( $update_data ) ) {
		$expiry = $update_data->expires;
		if ( 'lifetime' !== $expiry ) {
			$expiry = gmdate( 'M d, Y', strtotime( $expiry ) );
		} else {
			$expiry = 'never (lifetime)';
		}
		if ( isset( $update_data->license_limit ) ) {
			$license_limit = $update_data->license_limit;
		}
		if ( isset( $update_data->activations_left ) ) {
			$activations_left = $update_data->activations_left;
		}
		if ( isset( $update_data->license ) ) {
			$license = $update_data->license;
		}
		?>
	<div class="wrap">
		<h2><?php esc_html_e( 'Simple Podcast Press License Activator' ); ?></h2>
		<form method="post" action="options.php">
		
			<?php settings_fields( 'edd_sppress_license' ); ?>
			
			<table class="form-table">
				<tbody>
					<tr valign="top">	
						<th scope="row" valign="top">
							<?php esc_html_e( 'License Key' ); ?>
						</th>
						<td>
								<?php if ( false === $status && 'valid' === $license ) { ?>
							<input id="edd_sppress_license_key" name="edd_sppress_license_key" type="text" class="regular-text" disabled value="<?php esc_attr( $license_key ); ?>" />
								<?php } elseif ( 'expired' === $license ) { ?>
							<input id="edd_sppress_license_key" name="edd_sppress_license_key" type="text" class="regular-text" value="<?php esc_attr( $license_key ); ?>" />
								<?php } else { ?>
									<input id="edd_sppress_license_key" name="edd_sppress_license_key" type="text" class="regular-text" value="<?php echo esc_attr( $license_key ); ?>" />
								<?php } ?>
							<label class="description" for="edd_sppress_license_key"><?php esc_html_e( 'Enter your license key' ); ?></label>
								<br>
								<?php if ( false !== $status && 'valid' === $license ) { ?>
									<span style="color:green;"><i><?php esc_html_e( 'License key is active. Expires ' . $expiry . '. You have ' . $activations_left . '/' . $license_limit . ' activations left.' ); ?></i></span> <?php // phpcs:ignore WordPress.WP.I18n.NonSingularStringLiteralText ?>
								<?php } elseif ( 'expired' === $license ) { ?>
									<span style="color:red;"><i><?php esc_html_e( 'License key expired on ' . $expiry ); ?> <a href="https://simplepodcastpress.com/checkout/?edd_license_key=<?php echo esc_html( $license_key ); ?>&download_id=309" target="_blank">Click here to renew it. </a></i></span><?php // phpcs:ignore WordPress.WP.I18n.NonSingularStringLiteralText ?>
								<?php } else { ?>
									<span style="color:red;"><i><?php esc_html_e( 'Inactive or invalid license.' ); ?></i></span>
								<?php } ?>
						</td>
					</tr>
					<?php if ( false !== $license ) { ?>
						<tr valign="top">	
							<th scope="row" valign="top">
								<?php esc_html_e( 'Activate License' ); ?>
							</th>
							<td>
								<?php if ( false !== $status && 'valid' === $license ) { ?>
									<?php wp_nonce_field( 'edd_sppress_nonce', 'edd_sppress_nonce' ); ?>
										<input type="submit" class="button-secondary" name="spp_edd_license_deactivate" value="<?php esc_html_e( 'Deactivate License' ); ?>"/>
									<?php
								} else {
									wp_nonce_field( 'edd_sppress_nonce', 'edd_sppress_nonce' );
									?>
									<input type="submit" class="button-secondary" name="spp_edd_license_activate" value="<?php esc_html_e( 'Activate License' ); ?>"/>
								<?php } ?>
							</td>
						</tr>
					<?php } ?>
				</tbody>
			</table>			
		</form>
		<?php
	}
}

/**
 * Registers the license key option in the options table.
 *
 * @return void
 */
function edd_sppress_register_option() {
	// creates our settings in the options table.
	register_setting( 'edd_sppress_license', 'edd_sppress_license_key', 'edd_sppress_sanitize_license' );
}
add_action( 'admin_init', 'edd_sppress_register_option' );

/**
 * Sanitizes the license key before saving it in the options table.
 *
 * @param string $new_license The new license key.
 * @return string The sanitized license key.
 */
function edd_sppress_sanitize_license( $new_license ) {
	$old = get_option( 'edd_sppress_license_key' );
	if ( $old && $old !== $new_license ) {
		delete_option( 'sppress_ls' ); // new license has been entered, so must reactivate.
	}
	return $new_license;
}


/**
 * Activates the Simple Podcast Press license.
 *
 * This function handles the activation of the Simple Podcast Press license. It listens for the
 * activate button to be clicked, performs a security check, retrieves the license from the
 * database, and sends the activation request to the API. It then updates the license status
 * based on the response from the API.
 *
 * @return void
 */
function edd_sppress_activate_license() {
	// listen for our activate button to be clicked.
	if ( isset( $_POST['spp_edd_license_activate'] ) ) {

		// run a quick security check.
		if ( ! check_admin_referer( 'edd_sppress_nonce', 'edd_sppress_nonce' ) ) {
			return; // get out if we didn't click the Activate button.
		}

		$license_post = isset( $_POST['edd_sppress_license_key'] ) ? sanitize_text_field( wp_unslash( $_POST['edd_sppress_license_key'] ) ) : '';

		update_option( 'edd_sppress_license_key', $license_post );
		// retrieve the license from the database.
		$license = trim( get_option( 'edd_sppress_license_key' ) );

		// data to send in our API request.
		$api_params = array(
			'edd_action' => 'activate_license',
			'license'    => $license,
			'item_name'  => rawurlencode( SPPRESS_ITEM_NAME ), // the name of our product in EDD.
			'url'        => rawurlencode( home_url() ),
		);

		// Call the custom API.
		$response = wp_remote_get(
			add_query_arg( $api_params, esc_url_raw( SPPRESS_STORE_URL ) ),
			array(
				'timeout'   => 15,
				'sslverify' => false,
			)
		);
		update_option( 'lic_check_response_fail', $response );

		// make sure the response came back okay.
		if ( is_wp_error( $response ) ) {
			return false;
		}

		// decode the license data.
		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// $license_data->license will be either "valid" or "invalid"
		update_option( 'sppress_ls', $license_data->license );

	}
}
add_action( 'admin_init', 'edd_sppress_activate_license' );


/**
 * Deactivates the Simple Podcast Press license.
 *
 * This function handles the deactivation of the Simple Podcast Press license. It listens for the
 * deactivate button to be clicked, performs a security check, retrieves the license from the
 * database, and sends the deactivation request to the API. It then updates the license status
 * based on the response from the API.
 *
 * @return void
 */
function edd_sppress_deactivate_license() {

	// listen for our activate button to be clicked.
	if ( isset( $_POST['spp_edd_license_deactivate'] ) ) {

		// run a quick security check.
		if ( ! check_admin_referer( 'edd_sppress_nonce', 'edd_sppress_nonce' ) ) {
			return; // get out if we didn't click the Activate button.
		}

		// retrieve the license from the database.
		$license = trim( get_option( 'edd_sppress_license_key' ) );

		// data to send in our API request.
		$api_params = array(
			'edd_action' => 'deactivate_license',
			'license'    => $license,
			'item_name'  => rawurlencode( SPPRESS_ITEM_NAME ), // the name of our product in EDD.
			'url'        => rawurlencode( home_url() ),
		);

		// Call the custom API.
		$response = wp_remote_get(
			add_query_arg( $api_params, esc_url_raw( SPPRESS_STORE_URL ) ),
			array(
				'timeout'   => 15,
				'sslverify' => false,
			)
		);

		// make sure the response came back okay.
		if ( is_wp_error( $response ) ) {
			return false;
		}

		// decode the license data.
		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// $license_data->license will be either "deactivated" or "failed"
		if ( 'deactivated' === $license_data->license ) {
			delete_option( 'sppress_ls' );
		}
	}
}
add_action( 'admin_init', 'edd_sppress_deactivate_license' );


/**
 * Checks the validity of the Simple Podcast Press license.
 *
 * This function sends a request to the API to check the validity of the Simple Podcast Press license.
 * It retrieves the license key from the database, constructs the API request parameters, and handles
 * the response from the API. If the license is valid, it echoes 'valid'. If not, it echoes 'invalid'.
 *
 * @return bolem
 */
function edd_sppress_check_license() {

	global $wp_version;

	$license = trim( get_option( 'edd_sppress_license_key' ) );

	$api_params = array(
		'edd_action' => 'check_license',
		'license'    => $license,
		'item_name'  => rawurlencode( SPPRESS_ITEM_NAME ),
		'url'        => rawurlencode( home_url() ),

	);

	// Call the custom API.
	$response = wp_remote_get(
		add_query_arg( $api_params, esc_url_raw( SPPRESS_STORE_URL ) ),
		array(
			'timeout'   => 15,
			'sslverify' => false,
		)
	);

	if ( is_wp_error( $response ) ) {
		return false;
	}

	$license_data = json_decode( wp_remote_retrieve_body( $response ) );

	if ( 'valid' === $license_data->license ) {
		echo 'valid';
		exit;
		// this license is still valid.
	} else {
		echo 'invalid';
		exit;
		// this license is no longer valid.
	}
}